import { Mapper } from '@automapper/core';
import { InjectMapper } from '@automapper/nestjs';
import { Inject } from '@nestjs/common';
import { CommandHandler } from '@nestjs/cqrs';
import { Messager } from '../../adapters/secondaries/messager';
import { AdsRepository } from '../../adapters/secondaries/ads.repository';
import { CreateAdCommand } from '../../commands/create-ad.command';
import { CreateAdRequest } from '../dtos/create-ad.request';
import { Ad } from '../entities/ad';
import { IProvideParams } from '../interfaces/param-provider.interface';
import { DefaultParams } from '../types/default-params.type';
import { Address } from '../entities/address';

@CommandHandler(CreateAdCommand)
export class CreateAdUseCase {
  private readonly defaultParams: DefaultParams;
  private ad: Ad;
  constructor(
    private readonly _repository: AdsRepository,
    private readonly _messager: Messager,
    @InjectMapper() private readonly _mapper: Mapper,
    @Inject('ParamsProvider')
    private readonly defaultParamsProvider: IProvideParams,
  ) {
    this.defaultParams = defaultParamsProvider.getParams();
  }

  async execute(command: CreateAdCommand): Promise<Ad> {
    console.log('before mapping ');
    const entity: Ad = this._mapper.map(
      command.createAdRequest,
      CreateAdRequest,
      Ad,
    );
    typeof entity.monMargin === 'number'
      ? entity.monMargin
      : (entity.monMargin = this.defaultParams.MON_MARGIN);
    typeof entity.tueMargin === 'number'
      ? entity.tueMargin
      : (entity.tueMargin = this.defaultParams.TUE_MARGIN);
    typeof entity.wedMargin === 'number'
      ? entity.wedMargin
      : (entity.wedMargin = this.defaultParams.WED_MARGIN);
    typeof entity.thuMargin === 'number'
      ? entity.thuMargin
      : (entity.thuMargin = this.defaultParams.THU_MARGIN);
    typeof entity.friMargin === 'number'
      ? entity.friMargin
      : (entity.friMargin = this.defaultParams.FRI_MARGIN);
    typeof entity.satMargin === 'number'
      ? entity.satMargin
      : (entity.satMargin = this.defaultParams.SAT_MARGIN);
    typeof entity.sunMargin === 'number'
      ? entity.sunMargin
      : (entity.sunMargin = this.defaultParams.SUN_MARGIN);
    typeof entity.strict === 'boolean'
      ? entity.strict
      : (entity.strict = this.defaultParams.STRICT);

    if (typeof entity.addresses[0].position === 'undefined') {
      for (let i = 0; i < entity.addresses.length; i++) {
        entity.addresses[i].position = i;
      }
    }
    try {
      console.log('before ***********************************');
      console.log(entity);
      console.log('******************************************');
      this.ad = await this._repository.create(entity);
      // this._messager.publish('ad.create', JSON.stringify(ad));
      // this._messager.publish('logging.ad.create.info', JSON.stringify(ad));
      return this.ad;
    } catch (error) {
      let key = 'logging.ad.create.crit';
      if (error.message.includes('Already exists')) {
        key = 'logging.ad.create.warning';
      }
      this._messager.publish(
        key,
        JSON.stringify({
          command,
          error,
        }),
      );
      throw error;
    }
  }
}
/*
 repo error 
 PrismaClientValidationError: 
 Invalid `this._prisma[this._model].create()` invocation in
 /usr/src/app/src/modules/ad/adapters/secondaries/ads.repository.ts:15:51
 
   12 console.log(entity);
   13 console.log('*****************************************');
   14 try {
 → 15   const res = await this._prisma[this._model].create({
          data: {
            uuid: undefined,
            userUuid: '113e0000-0000-4000-a000-000000000000',
            driver: true,
            passenger: false,
            frequency: 'RECURRENT',
            fromDate: new Date('2023-01-15T00:00:00.000Z'),
            toDate: new Date('2023-08-01T00:00:00.000Z'),
            seatsDriver: 3,
            seatsPassenger: 0,
            strict: false,
            monMargin: 800,
            tueMargin: 900,
            wedMargin: 900,
            thuMargin: 900,
            friMargin: 900,
            satMargin: 900,
            sunMargin: 900,
            monTime: '04:40',
            tueTime: undefined,
            wedTime: undefined,
            thuTime: undefined,
            friTime: undefined,
            satTime: undefined,
            sunTime: undefined,
            addresses: [
              {
                lon: 48.68944549560547,
                lat: 6.176510334014893,
                houseNumber: '5',
                street: 'Avenue Foch',
                locality: 'Nancy',
                postalCode: '54000',
                country: 'France',
                position: 0
              },
              {
                lon: 48.85660171508789,
                lat: 2.3522000312805176,
                locality: 'Paris',
                postalCode: '75000',
                country: 'France',
                position: 1
              }
            ]
            ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
          },
          include: undefined
        })
 
 Argument addresses: Got invalid value 
 [
   {
     lon: 48.68944549560547,
     lat: 6.176510334014893,
     houseNumber: '5',
     street: 'Avenue Foch',
     locality: 'Nancy',
     postalCode: '54000',
     country: 'France',
     position: 0
   },
   {
     lon: 48.85660171508789,
     lat: 2.3522000312805176,
     locality: 'Paris',
     postalCode: '75000',
     country: 'France',
     position: 1
   }
 ]
 on prisma.createOneAd. Provided List<Json>, expected AddressCreateNestedManyWithoutAdInput:
 type AddressCreateNestedManyWithoutAdInput {
   create?: AddressCreateWithoutAdInput | List<AddressCreateWithoutAdInput> | AddressUncheckedCreateWithoutAdInput | List<AddressUncheckedCreateWithoutAdInput>
   connectOrCreate?: AddressCreateOrConnectWithoutAdInput | List<AddressCreateOrConnectWithoutAdInput>
   createMany?: AddressCreateManyAdInputEnvelope
   connect?: AddressWhereUniqueInput | List<AddressWhereUniqueInput>
 }
 
 
     at mi.validate (/usr/src/app/node_modules/@prisma/client/runtime/library.js:149:91)
     at tn.createMessage (/usr/src/app/node_modules/@prisma/client/runtime/library.js:166:1205)
     at /usr/src/app/node_modules/@prisma/client/runtime/library.js:179:10962
     at runInChildSpan (/usr/src/app/node_modules/@prisma/client/runtime/library.js:70:25817)
     at PrismaService._executeRequest (/usr/src/app/node_modules/@prisma/client/runtime/library.js:179:10951)
     at PrismaService._request (/usr/src/app/node_modules/@prisma/client/runtime/library.js:179:10484)
     at AdsRepository.create (/usr/src/app/src/modules/ad/adapters/secondaries/ads.repository.ts:15:19)
     at CreateAdUseCase.execute (/usr/src/app/src/modules/ad/domain/usecases/create-ad.usecase.ts:69:17)
     at AdController.createAd (/usr/src/app/src/modules/ad/adapters/primaries/ad.controller.ts:49:18)
     at /usr/src/app/node_modules/@nestjs/microservices/context/rpc-proxy.js:11:32
     at Object.Create (/usr/src/app/node_modules/@nestjs/microservices/server/server-grpc.js:148:40) {
   clientVersion: '4.13.0'
*/
